#include "dynamic_libs.h"

#ifdef _WIN32
#define WIN32_LEAN_AND_MEAN
#include <Windows.h>

VY_DLLEXPORT vy_dynlib vyOpenLib(const char *libname) {
    wchar_t libname_w[MAX_PATH];
    MultiByteToWideChar(CP_UTF8,
                        MB_PRECOMPOSED,
                        libname,
                        -1,
                        libname_w,
                        MAX_PATH);
    HMODULE mod = LoadLibraryW(libname_w);
    return (vy_dynlib)mod;
}

VY_DLLEXPORT void *vyGetSymbol(vy_dynlib lib, const char *symbol) {
    return (void*)GetProcAddress((HMODULE)lib, symbol);
}

VY_DLLEXPORT void vyCloseLib(vy_dynlib lib) {
    FreeLibrary((HMODULE)lib);
}

#elif defined(__linux__)
#include <dlfcn.h>

VY_DLLEXPORT vy_dynlib vyOpenLib(const char *libname) {
    return dlopen(libname, RTLD_NOW | RTLD_LOCAL);
}

VY_DLLEXPORT void *vyGetSymbol(vy_dynlib lib, const char *symbol) {
    return dlsym(lib, symbol);
}

VY_DLLEXPORT void vyCloseLib(vy_dynlib lib) {
    dlclose(lib);
}


#endif
