#include <stdarg.h>
#include <stdio.h>

#include "runtime.h"

#ifdef _WIN32
#define WIN32_LEAN_AND_MEAN
#include <Windows.h>
#endif

/* TODO(Kevin): Log to file, show error message box, ... */

static void DisplayErrorBox(const char *text)
{
#ifdef _WIN32
    WCHAR msg[256];
    MultiByteToWideChar(CP_UTF8,
                        MB_PRECOMPOSED,
                        text,
                        -1,
                        msg,
                        VY_ARRAY_COUNT(msg));
    MessageBoxW(NULL, msg, L"Error", MB_ICONERROR);
#endif
}

static void LogOut(const char *text)
{
#ifdef _WIN32
    WCHAR msg[256];
    MultiByteToWideChar(CP_UTF8,
                        MB_PRECOMPOSED,
                        text,
                        -1,
                        msg,
                        VY_ARRAY_COUNT(msg));
    OutputDebugStringW(msg);
#elif defined(__linux__)
    fprintf(stderr, "%s", text);
#endif
}

VY_DLLEXPORT void vyReportError(const char *subsystem, const char *fmt, ...) {
    char buf[256];

    int at = snprintf(buf, VY_ARRAY_COUNT(buf) - 1, "[%s] ", subsystem);
    va_list ap;
    va_start(ap, fmt);
    at += vsnprintf(&buf[at], VY_ARRAY_COUNT(buf) - at - 1, fmt, ap);
    va_end(ap);
    at += snprintf(&buf[at], VY_ARRAY_COUNT(buf) - at - 1, "\n");

    LogOut(buf);
    DisplayErrorBox(buf);
}

VY_DLLEXPORT void vyLog(const char *subsystem, const char *fmt, ...) {
    char buf[256];

    int at = snprintf(buf, VY_ARRAY_COUNT(buf) - 1, "[%s] ", subsystem);
    va_list ap;
    va_start(ap, fmt);
    at += vsnprintf(&buf[at], VY_ARRAY_COUNT(buf) - at - 1, fmt, ap);
    va_end(ap);
    at += snprintf(&buf[at], VY_ARRAY_COUNT(buf) - at - 1, "\n");

    LogOut(buf);
}