#ifndef VY_FIO_H
#define VY_FIO_H

#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>

#include "runtime.h"

enum
{
    VY_FILE_BUFFER_FLAG_FILE_NOT_FOUND = 0x1,
    VY_FILE_BUFFER_FLAG_READ_FAILED    = 0x2,
};

typedef struct {
    void *data;
    size_t size;
    uint32_t flags;
} vy_file_buffer;

inline bool vyWasFileBufferSuccessful(const vy_file_buffer *fb) {
    return fb->flags == 0;
}

/* used to identify a file (XXH3 hash of the path) */
typedef uint64_t vy_file_id;

typedef unsigned int vy_fio_handle;

typedef struct {
    unsigned int queue_size;
    unsigned int max_file_count;
} vy_fio_config;

VY_DLLEXPORT bool vyInitFIO(const vy_fio_config *config);

VY_DLLEXPORT void vyShutdownFIO(void);

VY_DLLEXPORT vy_file_id vyGetFileId(const char *path);

VY_DLLEXPORT vy_file_id vyGetFileIdFromSpan(vy_text_span path);

VY_DLLEXPORT vy_file_id vyAddFile(const char *path);

VY_DLLEXPORT vy_file_id vyAddFileFromSpan(vy_text_span path);

VY_DLLEXPORT const char *vyGetFilePath(vy_file_id fid);

VY_DLLEXPORT vy_fio_handle vyEnqueueRead(vy_file_id fid);

VY_DLLEXPORT void vyAbortFIO(vy_fio_handle fio);

VY_DLLEXPORT bool vyIsFIOFinished(vy_fio_handle fio);

VY_DLLEXPORT bool vyRetrieveReadBuffer(vy_fio_handle fio,
                                      vy_file_buffer *buffer);

VY_DLLEXPORT void vyFreeFileBuffer(vy_file_buffer buffer);

#endif
